use std::string::FromUtf8Error;

use axum::{http::StatusCode, response::IntoResponse};

pub type AppResult<R> = Result<R, AppError>;

#[derive(Debug, thiserror::Error)]
pub enum AppError {
    #[error(transparent)]
    ConfigError(#[from] envy::Error),

    #[error(transparent)]
    IOError(#[from] std::io::Error),

    #[error(transparent)]
    ReqwestError(#[from] reqwest::Error),

    #[error(transparent)]
    SledError(#[from] sled::Error),

    #[error(transparent)]
    BincodeError(#[from] bincode::Error),

    #[error(transparent)]
    Utf8Error(#[from] FromUtf8Error),

    #[error("Cancelled: {0}")]
    Cancelled(String),
}

impl IntoResponse for AppError {
    fn into_response(self) -> axum::response::Response {
        let err = self.to_string();

        (StatusCode::INTERNAL_SERVER_ERROR, err).into_response()
    }
}
